@@ -2,6 +2,8 @@
 
 import collections
 import datetime as dt
+import importlib.util
+import itertools
 import string
 from pathlib import Path
 from typing import Any, ClassVar, Dict, Iterable, List, Optional, Sequence, Set, Tuple
@@ -18,6 +20,8 @@ from mtg_ssm.mtg import util
 from mtg_ssm.scryfall.models import ScryCard, ScrySet
 from mtg_ssm.serialization import interface
 
+HAS_LXML = importlib.util.find_spec("lxml") is not None
+
 ALL_SETS_SHEET_HEADER: Sequence[str] = [
     "code",
     "name",
@@ -29,11 +33,10 @@ ALL_SETS_SHEET_HEADER: Sequence[str] = [
     "playsets",
     "count",
     "value",
-    "nonbulk",
 ]
 
 ALL_SETS_SHEET_TOTALS: Sequence[Optional[str]] = ["Total", None, None, None, None] + [
-    f"=SUM({c}3:{c}65535)" for c in "FGHIJK"
+    f"=SUM({c}3:{c}65535)" for c in "FGHIJ"
 ]
 
 
@@ -65,14 +68,13 @@ def create_all_sets(sheet: Worksheet, index: ScryfallDataIndex) -> None:
             f"=COUNTIF('{setcode}'!{_setsheet_col('have')}:{_setsheet_col('have')},\">=4\")",
             f"=SUM('{setcode}'!{_setsheet_col('have')}:{_setsheet_col('have')})",
             f"=SUM('{setcode}'!{_setsheet_col('value')}:{_setsheet_col('value')})",
-            f"=SUMIFS('{setcode}'!{_setsheet_col('value')}:{_setsheet_col('value')},'{setcode}'!{_setsheet_col('price')}:{_setsheet_col('price')},\">=1\")",
         ]
         sheet.append(row)
 
 
 def style_all_sets(sheet: Worksheet) -> None:
     """Apply styles to the all sets sheet."""
-    sheet.freeze_panes = sheet["C3"]
+    sheet.freeze_panes = "C3"
     col_width_hidden_format = [
         ("A", 8, False, None),
         ("B", 30, False, None),
@@ -84,7 +86,6 @@ def style_all_sets(sheet: Worksheet) -> None:
         ("H", 8, False, None),
         ("I", 7, False, None),
         ("J", 10, False, FORMAT_CURRENCY_USD_SIMPLE),
-        ("K", 10, False, FORMAT_CURRENCY_USD_SIMPLE),
     ]
     for col, width, hidden, number_format in col_width_hidden_format:
         cdim = sheet.column_dimensions[col]
@@ -94,15 +95,22 @@ def style_all_sets(sheet: Worksheet) -> None:
             cdim.number_format = number_format
 
 
-def create_haverefs(index: ScryfallDataIndex, cards: Sequence[ScryCard]) -> str:
-    """Create a reference to the have cells for printings in a single set."""
-    setcodes_and_rownums = sorted(
-        (c.set, index.id_to_setindex[c.id] + ROW_OFFSET) for c in cards
-    )
-    haverefs = [
-        f"'{setcode.upper()}'!A{rownum}" for setcode, rownum in setcodes_and_rownums
-    ]
-    return "+".join(haverefs)
+def create_haverefs(index: ScryfallDataIndex, setcode: str, cards: Sequence[ScryCard]) -> str:
+    """Create a reference to or sum of the have cell(s) for printings in a single set."""
+    setcode = setcode.upper()
+    rownums = sorted(index.id_to_setindex[card.id] + ROW_OFFSET for card in cards)
+    haverefs = []
+    # cluster contiguous rownums
+    for _, group in itertools.groupby(enumerate(rownums), lambda x: x[0] - x[1]):
+        cluster = [g[1] for g in group]
+        cells = [_setsheet_col("have") + str(r) for r in cluster]
+        if cells[0] == cells[-1]:
+            haverefs.append(f"'{setcode}'!{cells[0]}")
+        else:
+            haverefs.append(f"'{setcode}'!{cells[0]}:{cells[-1]}")
+    if len(haverefs) == 1 and ":" not in haverefs[0]:
+        return haverefs[0]
+    return f"SUM({','.join(haverefs)})"
 
 
 def get_references(
@@ -120,7 +128,7 @@ def get_references(
         if other_card.set not in exclude_sets:
             set_to_cards[other_card.set].append(other_card)
 
-    set_to_haveref = {k: create_haverefs(index, v) for k, v in set_to_cards.items()}
+    set_to_haveref = {k: create_haverefs(index, k, v) for k, v in set_to_cards.items()}
     if not set_to_haveref:
         return None
 
@@ -129,11 +137,15 @@ def get_references(
         set_to_haveref,
         key=lambda setcode: _card_set_sort_key(index.setcode_to_set[setcode]),
     ):
-        reference = 'IF({count}>0,"{setcode}: "&{count}&", ","")'.format(
-            setcode=card_set.upper(), count=set_to_haveref[card_set]
+        reference = (
+            f'IF({set_to_haveref[card_set]}>0,"{card_set.upper()}:"&{set_to_haveref[card_set]},"")'
         )
         references.append(reference)
-    return "=" + "&".join(references)
+    if not references:
+        return None
+    if len(references) == 1:
+        return "=" + references[0]
+    return f'=_xlfn.TEXTJOIN(", ",1,{",".join(references)})'
 
 
 ALL_CARDS_SHEET_HEADER = ["name", "have"]  # TODO: add list of sets
@@ -150,8 +162,11 @@ def create_all_cards(sheet: Worksheet, index: ScryfallDataIndex) -> None:
 
 def style_all_cards(sheet: Worksheet) -> None:
     """Apply styles to the all cards sheet."""
-    sheet.freeze_panes = sheet["B2"]
-    col_width_hidden = [("A", 24, False), ("B", 32, False)]
+    sheet.freeze_panes = "A2"
+    col_width_hidden = [
+        ("A", 28, False),
+        ("B", 48, False),
+    ]
     for col, width, hidden in col_width_hidden:
         cdim = sheet.column_dimensions[col]
         cdim.width = width
@@ -205,9 +220,7 @@ VALUE_TMPL = (
 ROW_OFFSET = 2
 
 
-def create_set_sheet(
-    sheet: Worksheet, collection: MagicCollection, setcode: str
-) -> None:
+def create_set_sheet(sheet: Worksheet, collection: MagicCollection, setcode: str) -> None:
     """Populate sheet with card information from a given set."""
     index = collection.oracle.index
 
@@ -235,7 +248,7 @@ def create_set_sheet(
 
 def style_set_sheet(sheet: Worksheet) -> None:
     """Apply styles to a set sheet."""
-    sheet.freeze_panes = sheet["E2"]
+    sheet.freeze_panes = "E2"
     col_width_hidden_format = [
         ("A", 5, False, None),
         ("B", 9, False, FORMAT_CURRENCY_USD_SIMPLE),
@@ -280,22 +293,22 @@ def rows_for_workbook(
 
 
 class XlsxDialect(interface.SerializationDialect):
-    """excel xlsx collection"""
+    """excel xlsx collection."""
 
     extension: ClassVar[str] = "xlsx"
     dialect: ClassVar[str] = "xlsx"
 
     def write(self, path: Path, collection: MagicCollection) -> None:
         """Write collection to an xlsx file."""
-        workbook = openpyxl.Workbook()
+        workbook = openpyxl.Workbook(write_only=HAS_LXML)
 
         all_sets_sheet = workbook.create_sheet()
-        create_all_sets(all_sets_sheet, collection.oracle.index)
         style_all_sets(all_sets_sheet)
+        create_all_sets(all_sets_sheet, collection.oracle.index)
 
         all_cards_sheet = workbook.create_sheet()
-        create_all_cards(all_cards_sheet, collection.oracle.index)
         style_all_cards(all_cards_sheet)
+        create_all_cards(all_cards_sheet, collection.oracle.index)
 
         setcodes = [
             s.code
@@ -307,9 +320,14 @@ class XlsxDialect(interface.SerializationDialect):
 
         for setcode in setcodes:
             set_sheet = workbook.create_sheet()
-            create_set_sheet(set_sheet, collection, setcode)
             style_set_sheet(set_sheet)
-        del workbook["Sheet"]
+            create_set_sheet(set_sheet, collection, setcode)
+
+        if not HAS_LXML:
+            # write_only mode does no create a default sheet but read/write mode does
+            # if we do not have lxml, we are in read/write mode and must delete the
+            # default sheet
+            del workbook["Sheet"]
         workbook.save(str(path))
 
     def read(self, path: Path, oracle: Oracle) -> MagicCollection:
